/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package caosdb.server;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

public class ServerProperties extends Properties {

  private static final long serialVersionUID = 1L;

  public static final String KEY_FILE_SYSTEM_ROOT = "FILE_SYSTEM_ROOT";
  public static final String KEY_DROP_OFF_BOX = "DROP_OFF_BOX";
  public static final String KEY_TMP_FILES = "TMP_FILES";
  public static final String KEY_USER_FOLDERS = "USER_FOLDERS";
  public static final String KEY_CHOWN_SCRIPT = "CHOWN_SCRIPT";
  public static final String KEY_AUTH_OPTIONAL = "AUTH_OPTIONAL";

  public static final String KEY_MYSQL_HOST = "MYSQL_HOST";
  public static final String KEY_MYSQL_PORT = "MYSQL_PORT";
  public static final String KEY_MYSQL_DATABASE_NAME = "MYSQL_DATABASE_NAME";
  public static final String KEY_MYSQL_USER_NAME = "MYSQL_USER_NAME";
  public static final String KEY_MYSQL_USER_PASSWORD = "MYSQL_USER_PASSWORD";
  public static final String KEY_MYSQL_SCHEMA_VERSION = "SQL_SCHEMA_VERSION";

  public static final String KEY_BASE_PATH = "BASE_PATH";
  public static final String KEY_FILE_POLICY = "FILE_POLICY";
  public static final String KEY_FILE_MESSAGES = "FILE_MESSAGES";
  public static final String KEY_CONTEXT_ROOT = "CONTEXT_ROOT";
  public static final String KEY_POLICY_COMPONENT = "POLICY_COMPONENT";

  public static final String KEY_SERVER_PORT_HTTPS = "SERVER_PORT_HTTPS";
  public static final String KEY_SERVER_PORT_HTTP = "SERVER_PORT_HTTP";

  public static final String KEY_HTTPS_ENABLED_PROTOCOLS = "HTTPS_ENABLED_PROTOCOLS";
  public static final String KEY_HTTPS_DISABLED_PROTOCOLS = "HTTPS_DISABLED_PROTOCOLS";
  public static final String KEY_HTTPS_ENABLED_CIPHER_SUITES = "HTTPS_ENABLES_CIPHER_SUITES";
  public static final String KEY_HTTPS_DISABLED_CIPHER_SUITES = "HTTPS_DISABLES_CIPHER_SUITES";

  public static final String KEY_CERTIFICATES_KEY_STORE_PATH = "CERTIFICATES_KEY_STORE_PATH";
  public static final String KEY_CERTIFICATES_KEY_STORE_PASSWORD =
      "CERTIFICATES_KEY_STORE_PASSWORD";
  public static final String KEY_CERTIFICATES_KEY_PASSWORD = "CERTIFICATES_KEY_PASSWORD";

  public static final String KEY_INITIAL_CONNECTIONS = "INITIAL_CONNECTIONS";

  public static final String KEY_MAX_CONNECTIONS = "MAX_CONNECTIONS";

  public static final String KEY_ITEMS_PER_PAGE = "ITEMS_PER_PAGE";

  public static final String KEY_MAIL_HANDLER_CLASS = "MAIL_HANDLER_CLASS";
  public static final String KEY_MAIL_TO_FILE_HANDLER_LOC = "MAIL_TO_FILE_HANDLER_LOC";

  public static final String KEY_ADMIN_NAME = "ADMIN_NAME";
  public static final String KEY_ADMIN_EMAIL = "ADMIN_EMAIL";
  public static final String KEY_BUGTRACKER_URI = "BUGTRACKER_URI";

  public static final String KEY_UNITS_FILE = "UNITS_FILE";

  public static final String KEY_SESSION_TIMEOUT_MS = "SESSION_TIMEOUT_MS";
  public static final String KEY_ACTIVATION_TIMEOUT_MS = "ACTIVATION_TIMEOUT_MS";

  public static final String KEY_CACHE_CONF_LOC = "CACHE_CONF_LOC";

  public static final String KEY_RULES_CACHE_CAPACITY = "RULES_CACHE_CAPACITY";
  public static final String KEY_SPARSE_ENTITY_CACHE_CAPACITY = "SPARSE_ENTITY_CACHE_CAPACITY";
  public static final String KEY_PROPERTIES_CACHE_CAPACITY = "PROPERTIES_CACHE_CAPACITY";
  public static final String KEY_PARENTS_CACHE_CAPACITY = "PARENTS_CACHE_CAPACITY";
  public static final String KEY_USER_ACCOUNT_CACHE_CAPACITY = "USER_ACCOUNT_CACHE_CAPACITY";
  public static final String KEY_GROUP_CACHE_CAPACITY = "GROUP_CACHE_CAPACITY";

  public static final String KEY_TRANSACTION_BENCHMARK_ENABLED = "TRANSACTION_BENCHMARK_ENABLED";

  public static final String KEY_INSERT_FILES_IN_DIR_ALLOWED_DIRS =
      "INSERT_FILES_IN_DIR_ALLOWED_DIRS"; // see
  // server/jobs/core/InsertFilesInDir.java

  public static final String KEY_SUDO_PASSWORD = "SUDO_PASSWORD";

  public static final String KEY_USER_SOURCES_INI_FILE = "USER_SOURCES_INI_FILE";

  public static final String KEY_NEW_USER_DEFAULT_ACTIVITY = "NEW_USER_DEFAULT_ACTIVITY";

  public static final String KEY_PASSWORD_STRENGTH_REGEX = "PASSWORD_STRENGTH_REGEX";

  public static final String KEY_QUERY_FILTER_ENTITIES_WITHOUT_RETRIEVE_PERMISSIONS =
      "QUERY_FILTER_ENTITIES_WITHOUT_RETRIEVE_PERMISSIONS";

  public static final String KEY_SERVER_NAME = "SERVER_NAME";

  public static final String KEY_SERVER_OWNER = "SERVER_OWNER";

  /**
   * This init_server_properties method reads the config file which contains key-value-pairs for
   * such variables like the user name of the database, the port the server will be listening on
   * etc.
   */
  public static Properties initServerProperties() {
    final Properties serverProperties = new Properties();
    final String basepath = System.getProperty("user.dir");
    serverProperties.setProperty(KEY_SERVER_OWNER, "");
    serverProperties.setProperty(KEY_SERVER_NAME, "CaosDB Server");
    serverProperties.setProperty(KEY_FILE_SYSTEM_ROOT, "CaosDBFileSystem/FileSystemRoot/");
    serverProperties.setProperty(KEY_DROP_OFF_BOX, "CaosDBFileSystem/DropOffBox/");
    serverProperties.setProperty(KEY_TMP_FILES, "CaosDBFileSystem/TMP/");
    serverProperties.setProperty(KEY_CHOWN_SCRIPT, "misc/chown_script/caosdb_chown_dropoffbox");
    serverProperties.setProperty(KEY_USER_SOURCES_INI_FILE, basepath + "/conf/usersources.ini");
    serverProperties.setProperty(KEY_USER_FOLDERS, "FALSE");
    serverProperties.setProperty(KEY_NEW_USER_DEFAULT_ACTIVITY, "INACTIVE");

    serverProperties.setProperty(KEY_MYSQL_HOST, "localhost");
    serverProperties.setProperty(KEY_MYSQL_PORT, "3306");
    serverProperties.setProperty(KEY_MYSQL_DATABASE_NAME, "db_2_0");
    serverProperties.setProperty(KEY_MYSQL_USER_NAME, "CaosDB");
    serverProperties.setProperty(KEY_MYSQL_USER_PASSWORD, "CaosDB");

    serverProperties.setProperty(KEY_MYSQL_SCHEMA_VERSION, "v2.0.30");

    serverProperties.setProperty(KEY_BASE_PATH, basepath);
    serverProperties.setProperty(KEY_CONTEXT_ROOT, "");
    serverProperties.setProperty(KEY_POLICY_COMPONENT, "enabled");

    serverProperties.setProperty(KEY_SERVER_PORT_HTTPS, "0");
    serverProperties.setProperty(KEY_SERVER_PORT_HTTP, "0");

    serverProperties.setProperty(KEY_HTTPS_ENABLED_PROTOCOLS, "TLSv1.2 TLSv1.1 TLSv1");
    serverProperties.setProperty(KEY_HTTPS_DISABLED_PROTOCOLS, "SSLv3 SSLv2Hello");
    serverProperties.setProperty(
        KEY_HTTPS_ENABLED_CIPHER_SUITES,
        "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA TLS_RSA_WITH_AES_256_CBC_SHA TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA TLS_ECDH_RSA_WITH_AES_256_CBC_SHA TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA TLS_RSA_WITH_AES_128_CBC_SHA TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA TLS_ECDH_RSA_WITH_AES_128_CBC_SHA TLS_ECDHE_ECDSA_WITH_RC4_128_SHA TLS_ECDHE_RSA_WITH_RC4_128_SHA TLS_ECDH_ECDSA_WITH_RC4_128_SHA TLS_ECDH_RSA_WITH_RC4_128_SHA TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA");
    serverProperties.setProperty(
        KEY_HTTPS_DISABLED_CIPHER_SUITES,
        "SSL_RSA_WITH_RC4_128_MD5 SSL_RSA_WITH_3DES_EDE_CBC_SHA SSL_RSA_WITH_RC4_128_SHA TLS_DHE_RSA_WITH_AES_256_CBC_SHA TLS_DHE_DSS_WITH_AES_256_CBC_SHA TLS_DHE_RSA_WITH_AES_128_CBC_SHA TLS_DHE_DSS_WITH_AES_128_CBC_SHA SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA");

    serverProperties.setProperty(KEY_INITIAL_CONNECTIONS, "1");

    serverProperties.setProperty(KEY_MAX_CONNECTIONS, "10");

    serverProperties.setProperty(KEY_ITEMS_PER_PAGE, "100");

    serverProperties.setProperty(KEY_SESSION_TIMEOUT_MS, "600000"); // 10 min
    serverProperties.setProperty(KEY_ACTIVATION_TIMEOUT_MS, "604800000"); // 7days

    serverProperties.setProperty(KEY_MAIL_HANDLER_CLASS, "caosdb.server.utils.mail.ToFileHandler");
    serverProperties.setProperty(KEY_MAIL_TO_FILE_HANDLER_LOC, "./OUTBOX");

    serverProperties.setProperty(KEY_ADMIN_NAME, "CaosDB Admin");
    serverProperties.setProperty(KEY_ADMIN_EMAIL, "");
    serverProperties.setProperty(KEY_BUGTRACKER_URI, "");

    serverProperties.setProperty(KEY_UNITS_FILE, basepath + "/src/main/java/caosdb/unit/si.units");

    serverProperties.setProperty(KEY_TRANSACTION_BENCHMARK_ENABLED, "true");
    serverProperties.setProperty(KEY_CACHE_CONF_LOC, "conf/cache.ccf");

    serverProperties.setProperty(KEY_RULES_CACHE_CAPACITY, "100");
    serverProperties.setProperty(KEY_SPARSE_ENTITY_CACHE_CAPACITY, "1000");
    serverProperties.setProperty(KEY_PROPERTIES_CACHE_CAPACITY, "1000");
    serverProperties.setProperty(KEY_PARENTS_CACHE_CAPACITY, "1000");
    serverProperties.setProperty(KEY_USER_ACCOUNT_CACHE_CAPACITY, "100");
    serverProperties.setProperty(KEY_GROUP_CACHE_CAPACITY, "100");

    serverProperties.setProperty(KEY_INSERT_FILES_IN_DIR_ALLOWED_DIRS, "");

    serverProperties.setProperty(KEY_SUDO_PASSWORD, "");
    serverProperties.setProperty(KEY_QUERY_FILTER_ENTITIES_WITHOUT_RETRIEVE_PERMISSIONS, "TRUE");

    try {
      final File confFile = new File(basepath + "/conf/server.conf");
      if (confFile.exists()) {
        final BufferedInputStream sp_in = new BufferedInputStream(new FileInputStream(confFile));
        serverProperties.load(sp_in);
        sp_in.close();
      } else {
        System.out.println("No config file found. Will use standard configuration.");
      }

    } catch (final IOException e) {
      System.out.println("Failed to read config file. Will use standard configuration.");
    }
    // this is due to the userfolders must have a users name...
    if (serverProperties.getProperty(KEY_USER_FOLDERS).equalsIgnoreCase("TRUE")) {
      serverProperties.setProperty(KEY_POLICY_COMPONENT, "enabled");
    }
    return serverProperties;
  }
}
