/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package caosdb.server.logging;

import caosdb.server.transaction.InsertLogRecordTransaction;
import caosdb.server.utils.CronJob;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.ErrorManager;
import java.util.logging.Handler;
import java.util.logging.LogRecord;

public class BackendLoggingHandler extends Handler {

  private static List<LogRecord> list = new LinkedList<LogRecord>();
  private static final Integer lock = 0;
  private static boolean isConnected = false;

  private static void setIsConnected(final boolean b) {
    isConnected = b;
  }

  @Override
  public void close() throws SecurityException {
    flush();
    setIsConnected(false);
  }

  @Override
  public void flush() {
    doFlush(getErrorManager());
  }

  private static void doFlush(final ErrorManager em) {
    List<LogRecord> toBeFlushed = null;
    synchronized (lock) {
      if (!isConnected) {
        return;
      }
      if (list.isEmpty()) {
        return;
      } else {
        toBeFlushed = list;
        list = new LinkedList<LogRecord>();
      }
    }
    writeToBackend(toBeFlushed, em);
  }

  private static void writeToBackend(final List<LogRecord> toBeFlushed, final ErrorManager em) {
    final InsertLogRecordTransaction t = new InsertLogRecordTransaction(toBeFlushed);
    try {
      t.execute();
    } catch (final Exception e) {
      em.error("Flushing LogRecords failed", e, 0);
    }
  }

  @Override
  public void publish(final LogRecord arg0) {
    synchronized (lock) {
      list.add(arg0);
    }
  }

  public static void start() {
    setIsConnected(true);
    new CronJob(
        "FLUSH SERVER LOGS",
        new Runnable() {

          @Override
          public void run() {
            doFlush(new ErrorManager());
          }
        },
        10);
  }
}
