/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package caosdb.server.utils;

import caosdb.server.CaosDBServer;
import caosdb.server.ServerProperties;
import caosdb.server.entity.Message;
import caosdb.server.entity.Message.MessageType;

public class ServerMessages {

  public static final Message ENTITY_HAS_BEEN_DELETED_SUCCESSFULLY =
      new Message(MessageType.Info, 10, "This entity has been deleted successfully.");

  public static final Message ENTITY_DOES_NOT_EXIST =
      new Message(MessageType.Error, 101, "Entity does not exist.");

  public static final Message ENTITY_HAS_UNQUALIFIED_PROPERTIES =
      new Message(MessageType.Error, 114, "Entity has unqualified properties.");

  public static final Message ENTITY_HAS_UNQUALIFIED_PARENTS =
      new Message(MessageType.Error, 116, "Entity has unqualified parents.");

  public static final Message PARSING_FAILED = new Message(MessageType.Error, 2, "Parsing failed.");

  public static final Message UNKNOWN_DATATYPE =
      new Message(MessageType.Error, 0, "Unknown datatype.");

  public static final Message UNKNOWN_IMPORTANCE =
      new Message(MessageType.Error, 0, "Unknown importance.");

  public static final Message ENTITY_HAS_NO_ID =
      new Message(MessageType.Error, 0, "Entity has no ID.");

  public static final Message REQUIRED_BY_PERSISTENT_ENTITY =
      new Message(
          MessageType.Error,
          0,
          "Entity is required by other entities which are not to be deleted.");

  public static final Message NO_DATATYPE =
      new Message(MessageType.Error, 110, "Property has no datatype.");

  public static final Message ENTITY_HAS_NO_DESCRIPTION =
      new Message(MessageType.Error, 0, "Entity has no description.");

  public static final Message ENTITY_HAS_NO_NAME =
      new Message(MessageType.Error, 0, "Entity has no name.");

  public static final Message OBLIGATORY_PROPERTY_MISSING =
      new Message(MessageType.Error, 0, "An obligatory property is missing.");

  public static final Message ENTITY_HAS_NO_PARENTS =
      new Message(MessageType.Error, 0, "Entity has no parents.");

  public static final Message NAME_DUPLICATES =
      new Message(MessageType.Error, 0, "Entity can not be identified due to name duplicates.");

  public static final Message ENTITY_HAS_NO_NAME_AND_NO_ID =
      new Message(MessageType.Error, 0, "Entity has no name and no ID.");

  public static final Message ENTITY_HAS_NO_PROPERTIES =
      new Message(MessageType.Error, 0, "Entity has no properties.");

  public static final Message REFERENCE_HAS_NO_REFID =
      new Message(MessageType.Error, 0, "Reference property has no refid.");

  public static final Message NO_TARGET_PATH =
      new Message(MessageType.Error, 0, "No target path specified.");

  public static final Message TARGET_PATH_NOT_ALLOWED =
      new Message(MessageType.Error, 0, "This target path is not allowed.");

  public static final Message TARGET_PATH_EXISTS =
      new Message(MessageType.Error, 0, "This target path does already exist.");

  public static final Message ENTITY_HAS_NO_UNIT =
      new Message(MessageType.Error, 0, "Entity has no unit.");

  public static final Message CANNOT_PARSE_VALUE =
      new Message(MessageType.Error, 0, "Cannot parse value.");

  public static final Message CHECKSUM_TEST_FAILED =
      new Message(MessageType.Error, 0, "Checksum test failed. File is corrupted.");

  public static final Message SIZE_TEST_FAILED =
      new Message(MessageType.Error, 0, "Size test failed. File is corrupted.");

  public static final Message CANNOT_CREATE_PARENT_FOLDER =
      new Message(MessageType.Error, 0, "Could not create parent folder in the file system.");

  public static final Message FILE_HAS_NOT_BEEN_UPLOAED =
      new Message(MessageType.Error, 0, "File has not been uploaded.");

  public static final Message THUMBNAIL_HAS_NOT_BEEN_UPLOAED =
      new Message(MessageType.Error, 0, "Thumbnail has not been uploaded.");

  public static final Message CANNOT_MOVE_FILE_TO_TARGET_PATH =
      new Message(MessageType.Error, 0, "Could not move file to it's target folder.");

  public static final Message CANNOT_PARSE_DATETIME_VALUE =
      new Message(
          MessageType.Error,
          0,
          "Cannot parse value to datetime format (yyyy-mm-dd'T'hh:mm:ss[.fffffffff][TimeZone]).");

  public static final Message CANNOT_PARSE_DOUBLE_VALUE =
      new Message(MessageType.Error, 0, "Cannot parse value to double.");

  public static final Message CANNOT_PARSE_INT_VALUE =
      new Message(MessageType.Error, 0, "Cannot parse value to integer.");

  public static final Message CANNOT_PARSE_BOOL_VALUE =
      new Message(
          MessageType.Error,
          0,
          "Cannot parse value to boolean (either 'true' or 'false, ignoring case).");

  public static final Message NOT_PERMITTED = new Message(MessageType.Error, 0, "Not permitted.");

  public static final Message CANNOT_CONNECT_TO_DATABASE =
      new Message(MessageType.Error, 0, "Could not connect to MySQL server.");

  public static final Message REQUEST_BODY_NOT_WELLFORMED =
      new Message(MessageType.Error, 0, "Request Body was not a well-formed xml.");

  public static final Message REQUEST_BODY_EMPTY =
      new Message(MessageType.Error, 0, "Request body was empty.");

  public static final Message MYSQL_PROCEDURE_EXCEPTION =
      new Message(
          MessageType.Error,
          0,
          "Please check if your MySQL has all required procedures installed.");

  public static final Message REQUEST_HAS_WRONG_ENCODING =
      new Message(
          MessageType.Error,
          0,
          "This error occurred while parsing the xml. Probably it has a wrong encoding.");

  public static final Message REQUEST_DOESNT_CONTAIN_EXPECTED_ELEMENTS =
      new Message(MessageType.Error, 0, "Request body didn't contain the expected Elements.");

  public static final Message FILE_NOT_IN_DROPOFFBOX =
      new Message(MessageType.Error, 0, "File is not in drop-off box.");

  public static final Message FILE_NOT_FOUND =
      new Message(MessageType.Error, 0, "File could not be found.");

  public static final Message CANNOT_MOVE_FILE_TO_TMP =
      new Message(MessageType.Error, 0, "Could not move file to tmp folder.");

  public static final Message CANNOT_READ_FILE =
      new Message(
          MessageType.Error,
          0,
          "Insufficient read permission for this file. Please make it readable.");

  public static final Message CANNOT_READ_THUMBNAIL =
      new Message(
          MessageType.Error,
          0,
          "Insufficient read permission for this file's thumbnail. Please make it readable.");

  public static final Message NO_FILE_REPRESENTATION_SUBMITTED =
      new Message(MessageType.Error, 0, "No file representation submitted.");

  public static final Message FORM_CONTAINS_UNSUPPORTED_CONTENT =
      new Message(MessageType.Error, 0, "The form contains unsupported content");

  public static final Message FILE_IS_EMPTY = new Message(MessageType.Error, 0, "File is empty.");

  public static final Message WARNING_OCCURED =
      new Message(
          MessageType.Error,
          128,
          "A warning occured while processing an entity with the strict flag.");

  public static final Message UNKNOWN_JOB = new Message(MessageType.Warning, 0, "Unknown job.");

  public static final Message NAME_IS_NOT_UNIQUE =
      new Message(
          MessageType.Error,
          152,
          "Name is already in use. Choose a different name or reuse an existing entity.");

  public static final Message ROLE_NAME_IS_NOT_UNIQUE =
      new Message(MessageType.Error, 1152, "Role name is already in use. Choose a different name.");

  public static final Message CANNOT_IDENTIFY_ENTITY_UNIQUELY =
      new Message(
          MessageType.Error, 0, "This entity cannot be identified uniquely due to name dublicates");

  public static final Message QUERY_EXCEPTION =
      new Message(MessageType.Error, 13, "This query finished with errors.");

  public static final Message LOGOUT_INFO =
      new Message(MessageType.Info, 201, "You have successfully logged out.");

  public static final Message ENTITY_IS_EMPTY =
      new Message(MessageType.Error, 0, "This entity is empty.");

  public static final Message TRANSACTION_ROLL_BACK =
      new Message(
          MessageType.Error,
          0,
          "An unknown error occured during the transaction and it was rolled back.");

  public static final Message FILE_UPLOAD_FAILED =
      new Message(MessageType.Error, 0, "The file upload failed for an unknown reason.");

  public static final Message ACCOUNT_CANNOT_BE_ACTIVATED =
      new Message(MessageType.Error, 0, "This account cannot be activated.");

  public static final Message ACCOUNT_DOES_NOT_EXIST =
      new Message(MessageType.Error, 0, "This account does not exist.");

  public static final Message ACCOUNT_CANNOT_BE_RESET =
      new Message(MessageType.Error, 0, "This account cannot be reset.");

  public static final Message UNKNOWN_UNIT =
      new Message(
          MessageType.Error,
          0,
          "Unknown unit. Values with this unit cannot be converted to other units when used in search queries.");

  public static final Message ACCOUNT_NAME_NOT_UNIQUE =
      new Message(MessageType.Error, 0, "This user name is yet in use. Please choose another one.");

  public static final Message ACCOUNT_HAS_BEEN_DELETED =
      new Message(MessageType.Info, 10, "This user has been deleted successfully.");

  public static final Message PLEASE_SET_A_PASSWORD =
      new Message(MessageType.Info, 0, "Please set a password.");

  public static final Message USER_HAS_BEEN_ACTIVATED =
      new Message(
          MessageType.Info,
          0,
          "User has been activated. You can now log in with your username and password.");

  public static final Message AUTHORIZATION_ERROR =
      new Message(MessageType.Error, 403, "You are not allowed to do this.");

  public static final Message REFERENCE_IS_NOT_ALLOWED_BY_DATATYPE =
      new Message(
          MessageType.Error,
          0,
          "Reference not qualified. The value of this Reference Property is to be a child of its data type.");

  public static final Message CANNOT_PARSE_ENTITY_ACL =
      new Message(MessageType.Error, 0, "Cannot parse EntityACL.");

  public static final Message ROLE_DOES_NOT_EXIST =
      new Message(MessageType.Error, 1104, "Role does not exist.");

  public static final Message ENTITY_NAME_DUPLICATES =
      new Message(MessageType.Error, 0, "This entity cannot be identified due to name duplicates.");

  public static final Message DATA_TYPE_NAME_DUPLICATES =
      new Message(
          MessageType.Error, 0, "This data type cannot be identified due to name duplicates.");

  public static final Message ENTITY_HAS_NO_NAME_OR_ID =
      new Message(
          MessageType.Error,
          0,
          "This entity cannot be identified as it didn't come with a name or id.");

  public static final Message EMAIL_NOT_WELL_FORMED =
      new Message(MessageType.Error, 0, "This email address is not RFC822 compliant.");

  public static final Message PASSWORD_TOO_WEAK =
      new Message(
          MessageType.Error,
          0,
          "This password is too weak. It should be longer than 8 characters and contain at least one number, one symbol, one uppercase letter and one lowercase letter. ");

  public static final Message AFFILIATION_ERROR =
      new Message(
          MessageType.Error, 0, "Affiliation is not defined for this child-parent constellation.");

  public static final Message QUERY_TEMPLATE_HAS_NO_QUERY_DEFINITION =
      new Message(MessageType.Error, 0, "This QueryTemplate has no query definition.");

  public static final Message QUERY_TEMPLATE_WITH_COUNT =
      new Message(
          MessageType.Error,
          0,
          "QueryTemplates may not be defined by 'COUNT...' queries for consistency reasons.");

  public static final Message QUERY_TEMPLATE_WITH_SELECT =
      new Message(
          MessageType.Error,
          0,
          "QueryTemplates may not be defined by 'SELECT... FROM...' queries for consistency reasons.");

  public static final Message QUERY_PARSING_ERROR =
      new Message(
          MessageType.Error,
          0,
          "An error occured during the parsing of this query. Maybe you use a wrong syntax?");

  public static final Message NAME_PROPERTIES_MUST_BE_TEXT =
      new Message(
          MessageType.Error,
          0,
          "A property which has 'name' as its parent must have a TEXT data type.");

  public static final Message PARENT_DUPLICATES_WARNING =
      new Message(
          MessageType.Warning,
          0,
          "This entity had parent duplicates. That is meaningless and only one parent had been inserted.");

  public static final Message PARENT_DUPLICATES_ERROR =
      new Message(
          MessageType.Error,
          0,
          "This entity had parent duplicates. Parent duplicates are meaningless and would be ignored (and inserted only once). But these parents had diverging inheritance instructions which cannot be processed.");

  public static final Message ATOMICITY_ERROR =
      new Message(
          MessageType.Error,
          12,
          "One or more entities are not qualified. None of them have been inserted/updated/deleted.");

  public static final Message NO_SUCH_ENTITY_ROLE(final String role) {
    return new Message(MessageType.Error, 0, "There is no such role '" + role + "'.");
  }

  public static Message UNKNOWN_ERROR(final String requestID) {
    return UNKNOWN_ERROR(
        requestID, CaosDBServer.getServerProperty(ServerProperties.KEY_BUGTRACKER_URI));
  }

  public static Message UNKNOWN_ERROR(final String sRID, final String reportTo) {
    final String description = "An unexpected server error has occurred.";
    String body = "";
    if (sRID != null) {
      body += "SRID = " + sRID;
    }
    if (reportTo != null) {
      body +=
          "\n\nPlease report this server error to "
              + reportTo
              + " and include the SRID into your report.";
    }

    return new Message(MessageType.Error, 500, description, (body.isEmpty() ? null : body));
  }

  public static final Message REQUIRED_BY_UNQUALIFIED =
      new Message(
          MessageType.Error, 192, "This entity cannot be deleted due to dependency problems");

  public static final Message ENTITY_HAS_INVALID_REFERENCE =
      new Message(MessageType.Error, 0, "This entity has an invalid reference.");

  public static final Message REFERENCED_ENTITY_DOES_NOT_EXIST =
      new Message(MessageType.Error, 235, "Referenced entity does not exist.");

  public static final Message REFERENCE_NAME_DUPLICATES =
      new Message(
          MessageType.Error, 0, "This reference cannot be identified due to name duplicates.");

  public static final Message DATATYPE_INHERITANCE_AMBIGUOUS =
      new Message(
          MessageType.Error,
          0,
          "The datatype which is to be inherited could not be detected due to divergent datatypes of at least two parents.");

  public static final Message DATA_TYPE_DOES_NOT_ACCEPT_COLLECTION_VALUES =
      new Message(
          MessageType.Error,
          0,
          "This datatype does not accept collections of values (e.g. Lists).");

  public static final Message CANNOT_PARSE_UNIT =
      new Message(MessageType.Error, 0, "This unit cannot be parsed.");
}
